from enum import Enum


class KeyType(Enum):
    ChoiceKey = 1
    StringKey = 2
    UTCTimeKey = 3
    TAITimeKey = 4
    EqCoordKey = 5
    SolarSystemCoordKey = 6
    MinorPlanetCoordKey = 7
    CometCoordKey = 8
    AltAzCoordKey = 9
    CoordKey = 10
    BooleanKey = 11
    CharKey = 12
    ByteKey = 13
    ShortKey = 14
    LongKey = 15
    IntKey = 16
    FloatKey = 17
    DoubleKey = 18
    ByteArrayKey = 19
    ShortArrayKey = 20
    LongArrayKey = 21
    IntArrayKey = 22
    FloatArrayKey = 23
    DoubleArrayKey = 24
    ByteMatrixKey = 25
    ShortMatrixKey = 26
    LongMatrixKey = 27
    IntMatrixKey = 28
    FloatMatrixKey = 29
    DoubleMatrixKey = 30
